<?php
require_once("dbconn.php");

function executeQuery($conn, $query, $types = null, $params = null) {
    // Prepare the statement
    $stmt = $conn->prepare($query);
    if ($stmt === false) {
        error_log("Prepare failed: (" . $conn->errno . ") " . $conn->error);
        return false;
    }

    // Bind parameters if provided
    if ($types !== null && $params !== null) {
        $stmt->bind_param($types, ...$params);
    }

    // Execute the statement
    $success = $stmt->execute();
    if (!$success) {
        error_log("Execute failed: (" . $stmt->errno . ") " . $stmt->error);
        return false;
    }

    // Handle SELECT queries
    if (strpos($query, "SELECT") === 0) {
        $result = $stmt->get_result();
        if ($result === false) {
            error_log("Get result failed: (" . $stmt->errno . ") " . $stmt->error);
            return false;
        }
        $stmt->close();
        return $result;
    } else {
        // Handle non-SELECT queries
        $affected_rows = $stmt->affected_rows;
        $stmt->close();
        return $affected_rows;
    }
}

function getDataFromPost($keys, $post) {
    $data = [];
    foreach ($keys as $key) 
	{
		$data[$key] = $post[$key];
    }
    return sanitizeAndValidate($data);
}

function sanitizeAndValidate($data) {
    $sanitizedData = [];
    foreach ($data as $key => $value) {
        $value = trim($value);
        $value = stripslashes($value);
        $value = htmlspecialchars_decode($value, ENT_QUOTES); // Decode HTML entities

        $sanitizedData[$key] = $value;
    }
    return $sanitizedData;
}

function fetchOriginalData($conn, $id) {
    $userType = isset($_SESSION["type"]) ? $_SESSION["type"] : null;
    $superUser = isset($_SESSION["superUser"]) ? $_SESSION["superUser"] : null;

    if (isset($_GET["p"])) {
        $page = $_GET["p"];

        if ($page === "cs") {
            $dbTable = "controller_standards";
        } elseif ($userType == 2 || isset($superUser)) {
            $dbTable = ($page === "ipep") ? "predata_ip" : 
                        (($page === "ipes") ? "predata_ew" : 
                        (($page === "ipfd") ? "predata_fd" : "table-error"));
        } elseif ($userType == 1 || $userType == 3) {
            $dbTable = ($page === "ipep") ? "input_production" : 
                        (($page === "ipes") ? "egg_weight" : 
                        (($page === "ipfd") ? "feed_delivery" : "table-error"));
        } else {
            $dbTable = "table-error";
        }
    } else {
        $dbTable = "table-error";
    }

    $query = "SELECT * FROM {$dbTable} WHERE id = ?";
    $stmt = mysqli_prepare($conn, $query);
    if ($stmt === false) {
        throw new Exception("Failed to prepare statement: " . $conn->error);
    }

    mysqli_stmt_bind_param($stmt, "i", $id);
    $executeResult = mysqli_stmt_execute($stmt);
    if ($executeResult === false) {
        mysqli_stmt_close($stmt);
        throw new Exception("Failed to execute statement: " . $conn->error);
    }

    $result = mysqli_stmt_get_result($stmt);
    if ($result === false) {
        mysqli_stmt_close($stmt);
        throw new Exception("Failed to get result: " . $conn->error);
    }

    $data = mysqli_fetch_assoc($result);
    mysqli_stmt_close($stmt);

    return $data;
}

function hasChanges($originalData, $newData) {
    foreach ($newData as $key => $value) {
        $originalKey = preg_replace('/_\d+$/', '', $key);

        if (!array_key_exists($originalKey, $originalData)) {
            continue; 
        }

        $originalValue = trim($originalData[$originalKey]);
        $newValue = trim($value);

        if ($originalValue !== $newValue) {
            return true; 
        }
    }
    return false; 
}

function getBatchNameOptions($conn) {
    $result = executeQuery($conn, "SELECT DISTINCT BatchName FROM epr_batch_details WHERE Status = 1 AND is_current = TRUE ORDER BY CheckInDate");
	
	if ($result === false) { 
		throw new Exception("Error executing query: " . $conn->error);
	} 

	$options = '<option value="">SELECT</option>';
	
    if ($result->num_rows > 0) {
        while($row = $result->fetch_assoc()) {
			$options .= '<option value="' . $row['BatchName'] . '">' . $row['BatchName'] . '</option>';
        }
    } else {
		$options = '<option value="" selected>SELECT</option>';
	}

    return $options;
}

function getBreedTypeOptions($conn) {
    $result = executeQuery($conn, "SELECT DISTINCT BreedType FROM controller_standards WHERE is_deleted = FALSE AND is_current = TRUE");
	
	if ($result === false) { 
		throw new Exception("Error executing query: " . $conn->error);
	} 

	$options = '<option value="">SELECT</option>';

    if ($result->num_rows > 0) {
        while($row = $result->fetch_assoc()) {
			$options .= '<option value="' . $row['BreedType'] . '">' . $row['BreedType'] . '</option>';
        }
    } else {
		$options = '<option value="" selected>SELECT</option>';
	}

    return $options;
}

function getCustomerNames($conn) {
    $result = executeQuery($conn, "SELECT customer_name FROM customer_names WHERE is_deleted = FALSE");
    
    if ($result === false) { 
        throw new Exception("Error executing query: " . $conn->error);
    } 

    $options = '<option value="">SELECT</option>';

	if (isset($_GET["p"]) && $_GET["p"] === "dr") {
        $options = '<option value="PG 7 - CEBU" selected>PG 7 - CEBU</option>';
    } else {
        if ($result->num_rows > 0) {
			while($row = $result->fetch_assoc()) {
				$options .= '<option value="' . $row['customer_name'] . '">' . $row['customer_name'] . '</option>';
			}
        } else {
			$options = '<option value="" selected>SELECT</option>';
		}
    } 

    return $options;
}

function generateDRNumber($conn) {
    $date = date("Ymd");
    $id = 1;
	$drsi_number = $_GET["p"] === "si" ?  "SI-" . $date . "-" . $id :  "DR-" . $date . "-" . $id;

    do {
		if($_GET["p"] === "si") {
			$result = executeQuery($conn, "SELECT COUNT(*) as count FROM sales_invoice WHERE si_number = ?", "s", [$drsi_number]);
		} else {
			$result = executeQuery($conn, "SELECT COUNT(*) as count FROM delivery_receipt WHERE dr_number = ?", "s", [$drsi_number]);
		}
		
		if ($result === false) { 
			throw new Exception("Error executing query: " . $conn->error);
		} 
		
        $row = $result->fetch_assoc();
        $count = $row["count"];

        if ($count > 0) {
            $id++;
			$drsi_number = $_GET["p"] === "si" ?  "SI-" . $date . "-" . $id :  "DR-" . $date . "-" . $id;
        }
    } while ($count > 0);

    return $drsi_number;
}

function numberFormatPrecision($number, $precision = 2, $separator = '.')
{
    $numberParts = explode($separator, $number);
    $response = $numberParts[0];
    if (count($numberParts)>1 && $precision > 0) {
        $response .= $separator;
        $response .= substr($numberParts[1], 0, $precision);
    }
    return $response;
}
?>